# SPDX-License-Identifier: MIT
"""
Compatibility module for those internal functions or classes not exposed from
the ``typing`` module even not exposed from the ``typing_extensions`` module.
"""

from typing_extensions import _AnnotatedAlias
